import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.NavigableSet;
import java.util.NoSuchElementException;

public final class Player extends Actor
{
    static double map(double x) { return 750 - x; }
    static int map(int x) { return 750 - x; }

    private InfinijumperWorld ijw;
    
    @Override public void addedToWorld(World world)
    {
        ijw = (InfinijumperWorld) world;
        ijw.pl = this;
        currentYLocation = initialYLocation = map(getY());
        initialTimeMillis = ijw.getTimeMillis();
        currentXLocation = getX();
    }
    
    public static final double g = -10, bounceFactor = 3.5, defaultVelocity = 40 * bounceFactor;
    
    private double velocity = 10.0;
    
    private int initialYLocation;
    private double tentativeYLocation, currentYLocation;
    private double currentXLocation;
    
    public void act() 
    {
        if(Greenfoot.isKeyDown("left"))
        {
            applyHorizontalForce(-0.4);
        }
        if(Greenfoot.isKeyDown("right"))
        {
            applyHorizontalForce(0.4);
        }
        
        calculateTentativeYLocation();
        
        //if(isFalling)
        if(tentativeYLocation <= currentYLocation)
        {
            Platform p = getFirstBlockingPlatform();
            if(p!=null)
            {
                p.bounceOff(this);
            }
        }
        currentYLocation = tentativeYLocation;
        
        setLocation((int) Math.round(currentXLocation), map((int) Math.round(currentYLocation)));
    }
    
    public void applyHorizontalForce(double force)
    {
        currentXLocation += force;
    }
    
    private long initialTimeMillis;
    
    private void calculateTentativeYLocation()
    {
        long interval = ijw.getTimeMillis() - initialTimeMillis;
        double intervalSeconds = (double) interval / 1000;
        tentativeYLocation = initialYLocation + velocity*intervalSeconds + intervalSeconds*intervalSeconds*g*bounceFactor;
    }
    
    public void reset(double newVelocity, Platform p)
    {
        initialTimeMillis = ijw.getTimeMillis();
        tentativeYLocation = initialYLocation = p.getYLocation() + 15 + 1;
        velocity = newVelocity;
    }
    
    public void reset(Platform p)
    {
        reset(defaultVelocity, p);
    }
    
    private Platform getFirstBlockingPlatform()
    {
        MutablePlatform current = new MutablePlatform((int) Math.floor(currentYLocation - 15)), tentative = new MutablePlatform((int) Math.floor(tentativeYLocation - 15));
        NavigableSet<Platform> ns = ijw.csls.subSet(tentative, true, current, true);
        
        Platform last;
        try
        {
            last = ns.last();
            if(Math.abs(last.getX() - currentXLocation) > (last.getWidth() / 2) + 15)
                last = null;            
        }
        catch(NoSuchElementException n)
        {
            last = null;
        }
        
        return last;
    }
}

class MutablePlatform extends Platform
{
    public MutablePlatform(int i)
    {
        yLocation = i;
    }
}